/*
 * HybServ TS Services, Copyright (C) 1998-1999 Patrick Alken
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: proxy.c,v 1.0.0 1999/06/23 15:44:22 wnder Exp $
 */

#include <unistd.h>
#include <assert.h>
#include <errno.h>
#include <string.h>

#include "client.h"
#include "config.h"
#include "dcc.h"
#include "defs.h"
#include "log.h"
#include "misc.h"
#include "operserv.h"
#include "proxy.h"
#include "settings.h"
#include "sock.h"

#ifdef HAVE_PTHREADS

/*
 * How many proxy queries we are currently engaged in -
 * If this exceeds MaxProxies, we stop doing proxy queries
 * for a bit. The idea is if a server netbursts, we don't
 * want a thousand proxy threads running.
 */
int ProxyHits = 0;

/*
Proxy()
 This routine will connect to the hostname that "nickname" is
using at PROXYPORT. If it finds the string "WinGate>" it will
assume the user is using an insecure proxy and notify +y users.
*/

void
Proxy(struct Luser *lptr)

{
  int sockfd,
      ret;
  char inbuf[MAXLINE];

  assert(lptr != 0);

  debug("BEFORE: Proxy = %d\n", ProxyHits);
  if (ProxyHits >= MaxProxies)
    return;

  ++ProxyHits;

  debug("AFTER: Proxy = %d\n", ProxyHits);

  if ((sockfd = ConnectHost(lptr->hostname, PROXYPORT)) >= 0)
  {
    /*
     * Connection successful
     */
    memset(inbuf, 0, sizeof(inbuf));
    if ((ret = read(sockfd, inbuf, MAXLINE - 1)) < 0)
    {
      putlog(LOG2,
        "Proxy error from host %s: %s",
        lptr->hostname,
        strerror(errno));

      --ProxyHits;
      return;
    }

    /*
     * The first read() would have picked up the special
     * telnet IAC codes, so we need to do a second read()
     * to see if there is a WinGate> prompt
     */
    ret = read(sockfd, inbuf, MAXLINE - 1);

    /*
     * An open wingate will display the prompt:
     * "WinGate>"
     * An open wingate that is too busy will display the prompt:
     * "Too many connected users - try again later"
     */
    if (!strncasecmp(inbuf, "WinGate>", 8) ||
        !strncasecmp(inbuf, "Too many", 8))
    {
      /*
       * We found a wingate
       */
      putlog(LOG2,
        "Insecure proxy: %s!%s@%s",
        lptr->nick,
        lptr->username,
        lptr->hostname);

      SendUmode(OPERUMODE_Y,
        "*** Insecure proxy: %s[%s@%s]",
        lptr->nick,
        lptr->username,
        lptr->hostname);

      /*
       * Mark them as running an insecure proxy
       */
      lptr->flags |= L_PROXY;
    }

    close (sockfd);
  }

  --ProxyHits;

  return;
} /* Proxy() */

/*
IsProxy()
 Determine if lptr is running an insecure proxy.
Return:
 1 if they are running a proxy
 0 if not
*/

int
IsProxy(struct Luser *lptr)

{
  if (!lptr)
    return (0);

  return (lptr->flags & L_PROXY);
} /* IsProxy() */

#endif /* HAVE_PTHREADS */
